/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew;

import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;

import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;

/**
 * Stellt Methoden zur Arbeit mit den TIC-Info (XML) Dateien bereit.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class XMLTools {

	/**
	 * Konvertiert eine Zeitangabe in einen String entsprechend der TIC-Info
	 * Spezifikation.
	 *
	 * @param millisec
	 *            Zeit
	 * @return Datum/Zeitangabe im Format yyyy-MM-dd'T'HH:mm:ss
	 */
	public static String getTICDateTime(final long millisec) {
		final SimpleDateFormat sdf = new SimpleDateFormat();
		sdf.applyPattern("yyyy-MM-dd'T'HH:mm:ss");
		sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
		return sdf.format(new Date(millisec));
	}

	/**
	 * Konvertiert eine Zeit aus einer TIC-Info Datei in einen Zeitwert.
	 *
	 * @param zeittext
	 *            Zeitangabe aus TIC-Info-Datei
	 * @return Zeitwert (Millisekunden seit 01.01.1970)
	 * @throws ParseException
	 *             wenn das Format des Textes nicht der erwarteten Struktur
	 *             entspricht.
	 */
	public static long getTICDateTime(final String zeittext) throws ParseException {
		final SimpleDateFormat sdf = new SimpleDateFormat();
		sdf.applyPattern("yyyy-MM-dd'T'HH:mm:ss");
		sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
		return sdf.parse(zeittext).getTime();
	}

	/**
	 * Gibt ein {@link Document} als XML-Datei aus.
	 *
	 * @param doc
	 *            das auszugebende Dokument
	 * @param datei
	 *            Datei die zu erzeugenden Datei
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	public static void schreibeXMLDatei(final Document doc, final File datei) throws TIC3Exception {
		try {
			final Source source = new DOMSource(doc);

			final Result result = new StreamResult(datei);

			final Transformer xformer = TransformerFactory.newInstance().newTransformer();
			// xformer.setOutputProperty(OutputKeys.INDENT, "yes");
			// xformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "no");
			xformer.setOutputProperty(OutputKeys.ENCODING, "utf-8");
			xformer.transform(source, result);
		} catch (final TransformerConfigurationException e) {
			throw new TIC3Exception(e.getMessage());
		} catch (final TransformerException e) {
			throw new TIC3Exception(e.getMessage());
		}
	}

	/**
	 * Konstruktor.
	 */
	protected XMLTools() {
		throw new UnsupportedOperationException(); // prevents calls from
		// subclass
	}

}
