/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;

/**
 * Externe Verkehrsmeldung.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id$
 */

public interface ExterneMeldung {

	/**
	 * Datenverteileranmeldung f&uuml;r Meldungsdaten.
	 *
	 * @param dav
	 *            die Datenverteilerverbindung
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	void anmeldenDav(final ClientDavInterface dav) throws TIC3Exception;

	/**
	 * Datenverteilerabmeldung f&uuml;r Meldungsdaten.
	 *
	 * @param dav
	 *            die Datenverteilerverbindung
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	void abmeldenDavEmpfang(final ClientDavInterface dav) throws TIC3Exception;

	/**
	 * Erzeugt eine PID aus den Meldungsdaten.
	 *
	 * @return PID
	 */
	String bildePid();

	/**
	 * Gibt das zugeh&ouml;rige Systemobjekt zur&uuml;ck.
	 *
	 * @return zugeh&ouml;riges Systemobjekt
	 */
	SystemObject getSystemObject();

	/**
	 * Setzt das zugeh&ouml;rige Systemobjekt.
	 * 
	 * @param object
	 *            {@link SystemObject}
	 */
	void setSystemObject(SystemObject object);

	/**
	 * Publiziert die Daten zum DaV.
	 *
	 * @param dav
	 *            Datenverteilerverbindung
	 * @param lmsParameter
	 *            LMSParameter
	 * @throws TIC3Exception
	 *             bei Ausnahmen
	 */
	void publiziereDav(final ClientDavInterface dav, LMSParameter lmsParameter) throws TIC3Exception;

	/**
	 * Gibt den zugeh&ouml;rigen DAV-Typ zur&uuml;ck.
	 * 
	 * @return DAV-Typ
	 */
	String getDavTyp();

	/**
	 * Gibt die Daten der Meldung zur&uuml;ck.
	 * 
	 * @return {@link ExterneMeldungsDaten}
	 */
	ExterneMeldungsDaten getMeldungsDaten();

	/**
	 * Setzt die Daten der Meldung.
	 * 
	 * @param daten
	 *            {@link ExterneMeldungsDaten}
	 */
	void setMeldungsDaten(ExterneMeldungsDaten daten);

	/**
	 * Gibt die eindeutige ID der Meldung zur&uuml;ck.
	 * 
	 * @return eindeutige ID
	 */
	String getID();
}
