/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;

/**
 * Repr&auml;sentiert ein ganzzahliges TIC3-Infoelement.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id$
 */

public class TMCInteger extends AbstractTIC3InfoElement {

	/**
	 * Wert.
	 */
	private Integer wert;

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public TMCInteger(final String xmlName, final String davName) {
		super(xmlName, davName);
	}

	/**
	 * Erzeugt ein neues Objekt mit Standardwert.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 * @param defaultwert
	 *            Standardwert
	 */
	public TMCInteger(final String xmlName, final String davName, final int defaultwert) {
		super(xmlName, davName);
		wert = defaultwert;
	}

	@Override
	public void getDavDaten(final Data davdata) throws TIC3Exception {
		if (davdata.isPlain()) {
			wert = davdata.asUnscaledValue().intValue();
		} else {
			wert = davdata.getUnscaledValue(getDavAttributName()).intValue();
		}
	}

	@Override
	public void getXmlDaten(final Node node) throws TIC3Exception {
		wert = Integer.parseInt(node.getTextContent());
	}

	@Override
	public void setDavDaten(final Data davdata) throws TIC3Exception {
		if (getWert() == null) {
			return;
		}

		if (davdata.isPlain()) {
			davdata.asUnscaledValue().set(getWert());
		} else {
			final Data item = davdata.getItem(getDavAttributName());
			if (item.isArray()) {
				final Data ddata;
				final Array array = item.asArray();
				array.setLength(1);
				ddata = array.getItem(0);
				ddata.asUnscaledValue().set(getWert());
			} else {
				item.asUnscaledValue().set(getWert());
			}
		}
	}

	@Override
	public void setXmlDaten(final Element element) {
		element.setTextContent(String.valueOf(wert));
	}

	/**
	 * Gibt den Wert zur&uuml;ck.
	 * 
	 * @return Wert
	 */
	public Integer getWert() {
		return wert;
	}

	/**
	 * Setzt den Wert.
	 * 
	 * @param wert
	 *            Wert
	 */
	public void setWert(final int wert) {
		this.wert = new Integer(wert);
	}

	@Override
	public Element erzeugeDocumentElement(final Document doc, final Element parent) throws TIC3Exception {
		if (getWert() == null) {
			return null;
		}

		return super.erzeugeDocumentElement(doc, parent);
	}

	@Override
	public String toString() {
		return getClass().getSimpleName() + " '" + getXmlElementName() + "': name='" + getNameAttribut().getXmlValue()
				+ "', Wert: '" + wert + "'";
	}
}
