/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lmstic3.vew.daten.tic.elemente;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.LMSParameter;
import de.bsvrz.kex.lmstic3.vew.daten.tic.AbstractTIC3InfoElement;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;

/**
 * TIC3-Lokationsinformationen mit TIC3-Lokationen.
 * 
 * <p>
 * Entspricht TIC3 Typ 'TIC3LocationType' (aktuell ohne EdgeTyp).
 * </p>
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id$
 */
public class TIC3TICLocation extends AbstractTIC3InfoElement {

	/**
	 * TIC3 Info XML Element: TIC3Location (T3L), A list of network references
	 * per network.
	 */
	public static final String XML_ELEMENT_TICLOC = "Tic3Location";

	/**
	 * DAV-Attribut.
	 */
	public static final String DAV_ATTR_TICLOC = "TIC3TicLocation";

	/**
	 * TIC3 Liste von Umrissen.
	 * <p>
	 * TIC Info XML Element: Shape (SHP), A list of shapes.
	 * </p>
	 */
	private List<TIC3ShapeType> shapes = new ArrayList<>();

	/**
	 * Erzeugt ein neues Objekt.
	 */
	public TIC3TICLocation() {
		super(XML_ELEMENT_TICLOC, null);

		/**
		 * TIC3 Netzkennung. TIC Info XML Element: NetworkIdentifier (NID), The
		 * identifier of the network. The network version is specified by the
		 * combination of the network identifier and the network release date. A
		 * reference to "Network".
		 */
		addKindElement(new TIC3Text("NetworkIdentifier", "TIC3Netzkennung", "value"));

		/**
		 * TIC3 Netzgebiet. TIC Info XML Element: NetworkArea (NAR), Information
		 * about an area such as the reference and description.
		 */
		addKindElement(new TIC3NetzGebiet());

		/**
		 * TIC3 Netzerzeugungszeit.
		 * <p>
		 * TIC Info XML Element: NetworkCreateTime (NVR), The date and time when
		 * the network was created, used as the release date of the identified
		 * network. The network version is specified by the combination of the
		 * network identifier and the network release date.
		 * </p>
		 */
		addKindElement(new TIC3Zeitstempel("NetworkCreateTime", "TIC3Netzerzeugungszeit"));
	}

	@Override
	public void publizieren(final Data davdata, final LMSParameter parameter) throws TIC3Exception {
		super.publizieren(davdata, parameter);

		// Shapes
		final Data.Array array = davdata.getArray("TIC3Shape");
		final int anzahl = shapes.size();
		array.setLength(anzahl);
		for (int i = 0; i < anzahl; i++) {
			shapes.get(i).publizieren(array.getItem(i), parameter);
		}
	}

	@Override
	public void einlesenXML(final Node node) throws TIC3Exception {
		super.einlesenXML(node);
		final NodeList nl = node.getChildNodes();

		for (int i = 0; i < nl.getLength(); i++) {
			final Node n = nl.item(i);
			if (n.getNodeName().equals("Shape")) {
				final TIC3ShapeType shape = new TIC3ShapeType("", "");
				shape.einlesenXML(n);
				shapes.add(shape);
			}
		}
	}

	@Override
	public String toString() {
		return this.getClass().getSimpleName();
	}
}
