/*
 * Segment 2 (KEx), SWE LMS-TIC3
 * Copyright (C) 2016 BitCtrl Systems GmbH 
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */


package de.bsvrz.kex.lmstic3.vew.daten.tmc;

import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TIC3Exception;
import de.bsvrz.kex.lmstic3.vew.daten.tic.TMCInteger;

/**
 * TMC-Priorität.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 * @version $Id$     
 */

public class TMCPrioritaet extends TMCInteger {

	/**
	 * Enumerator TMC-Priorit&auml;t.
	 */
	public enum TMCPrioritaetEnum {
		
		/** Unbekannt. */
		Unbekannt("Unbekannt", -1),

		/** Normal. */
		Normal("Normal", 0),

		/** Eilig. */
		Eilig("Eilig", 1),

		/** Sehr eilig. */
		SehrEilig("Sehr eilig", 2);

		// ??? name="Höchste Gefahr" wert="3"
		
		/**
		 * Gibt eine passende Priorit&auml;t-Instanz zur&uuml;ck.
		 *
		 * @param id
		 *            numerischer Wert
		 * @return zum numerischen Wert passende Instanz
		 * @throws TIC3Exception
		 *             bei ung&uuml;ltigem Wert
		 */
		public static TMCPrioritaetEnum getInstance(final int id) throws TIC3Exception {
			switch (id) {
			case -1:
				return Unbekannt;
			case 0:
				return Normal;
			case 1:
				return Eilig;
			case 2:
				return SehrEilig;

			default:
				throw new TIC3Exception("Keine TMC-Priorität mit id=" + id + " definiert.");
			}
		}

		/**
		 * textueller Wert.
		 */
		private final String value;

		/**
		 * numerischer Wert.
		 */
		private int id;

		/**
		 * Privater Konstruktor.
		 *
		 * @param value
		 *            textueller Wert
		 * @param id
		 *            numerischer Wert
		 */
		TMCPrioritaetEnum(final String value, final int id) {
			this.value = value;
			this.id = id;
		}

		/**
		 * Gibt den numerischen Wert zur&uuml;ck.
		 *
		 * @return numerischer Wert der Priorit&auml;t
		 */
		public int getId() {
			return id;
		}

		/**
		 * Gibt den textuellen Wert zur&uuml;ck.
		 *
		 * @return textueller Wert der Priorit&auml;t
		 */
		public String getValue() {
			return value;
		}
	}

	/**
	 * Priorit&auml;t.
	 */
	private TMCPrioritaetEnum prio = TMCPrioritaetEnum.Unbekannt;
	

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public TMCPrioritaet(final String xmlName, final String davName) {
		super(xmlName, davName);
	}

	@Override
	public void getDavDaten(final Data davdata) throws TIC3Exception {
		super.getDavDaten(davdata);
		prio = TMCPrioritaetEnum.getInstance(getWert());
	}

	@Override
	public void getXmlDaten(final Node node) throws TIC3Exception {
		prio = TMCPrioritaetEnum.getInstance(Integer.parseInt(node.getTextContent()));
	}

	@Override
	public void setDavDaten(final Data davdata) throws TIC3Exception {
		davdata.getUnscaledValue(getDavAttributName()).set(prio.getId());
	}
}
